/*
 * Copyright (c) 1995, 1996, 1997, 1998 Kungliga Tekniska Hgskolan
 * (Royal Institute of Technology, Stockholm, Sweden).
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by the Kungliga Tekniska
 *      Hgskolan and its contributors.
 * 
 * 4. Neither the name of the Institute nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE INSTITUTE AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE INSTITUTE OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

/*
 * The interface for the file-cache.
 */

/* $Id: fcache.h,v 1.20 1998/02/26 20:19:30 lha Exp $ */

#ifndef _FCACHE_H_
#define _FCACHE_H_

#include <xfs_message.h>
#include <fcntl.h>
#include <cred.h>

/*
 * For each entry in the filecache we save the rights of NACCESS
 * users.
 */ 

#define NACCESS 8

typedef struct {
     pag_t cred;
     u_long access;
} AccessEntry;

enum Access { AREAD   = 0x01,
	      AWRITE  = 0x02,
	      AINSERT = 0x04,
	      ALIST   = 0x08, 
	      ADELETE = 0x10,
	      ALINK   = 0x20,
	      AADMIN  = 0x40 };

typedef struct {
    struct Lock lock;
    VenusFid fid;
    u_long host;		/* the source of this entry */
    AFSFetchStatus status;	/* Removed unused stuff later */
    AFSCallBack callback;
    AFSVolSync volsync;	/* XXX - make smaller? */
    AccessEntry acccache[NACCESS];
    u_long anonaccess;
    unsigned nwriters;
    unsigned nreaders;
    ino_t inode;		/* right now only an index */
    struct {
	unsigned usedp : 1;
	unsigned attrp : 1;
	unsigned datap : 1;
	unsigned attrusedp : 1;
	unsigned datausedp : 1;
	unsigned extradirp : 1;
	unsigned mountp : 1;
    } flags;
    u_int tokens;
    VenusFid parent;
    VenusFid realfid;
    Listitem *listentry;
    VolCacheEntry *volume;
} FCacheEntry;

void
fcache_init (const char *cachedir,
	     u_long alowvnodes,
	     u_long ahighvnodes,
	     u_long alowbytes,
	     u_long ahighbytes);

int
fcache_reinit(u_long alowvnodes,
	      u_long ahighvnodes,
	      u_long alowbytes,
	      u_long ahighbytes);

void
fcache_purge_volume (VenusFid fid);

void
fcache_purge_host (u_long host);

void
fcache_stale_entry (VenusFid fid, AFSCallBack callback);

int
fcache_file_name (FCacheEntry *entry, char *s, size_t len);

int
fcache_extra_file_name (FCacheEntry *entry, char *s, size_t len);

int
fcache_open_file (FCacheEntry *entry, int flag, mode_t mode);

int
fcache_open_extra_dir (FCacheEntry *entry, int flag, mode_t mode);

int
write_data (FCacheEntry *entry, CredCacheEntry *ce);

int
truncate_file (FCacheEntry *entry, off_t size, CredCacheEntry *ce);

int
write_attr (FCacheEntry *entry, const AFSStoreStatus *status,
	    CredCacheEntry *ce);

int
create_file (FCacheEntry *dir_entry,
	     char *name, AFSStoreStatus *store_attr,
	     VenusFid *child_fid, AFSFetchStatus *fetch_attr,
	     CredCacheEntry *ce);

int
create_directory (FCacheEntry *dir_entry,
		  char *name, AFSStoreStatus *store_attr,
		  VenusFid *child_fid, AFSFetchStatus *fetch_attr,
		  CredCacheEntry *ce);

int
create_symlink (FCacheEntry *dir_entry,
		char *name, AFSStoreStatus *store_attr,
		VenusFid *child_fid, AFSFetchStatus *fetch_attr,
		char *contents,
		CredCacheEntry *ce);

int
create_link (FCacheEntry *dir_entry,
	     char *name,
	     FCacheEntry *existing_entry,
	     CredCacheEntry *ce);

int
remove_file (FCacheEntry *dire, const char *name, CredCacheEntry *ce);

int
remove_directory (FCacheEntry *dire, const char *name, CredCacheEntry *ce);

int
rename_file (FCacheEntry *old_dir,
	     const char *old_name,
	     FCacheEntry *new_dir,
	     const char *new_name,
	     CredCacheEntry *ce);

int
getroot (VenusFid *res, CredCacheEntry *ce);

int
fcache_get (FCacheEntry **res, VenusFid fid, CredCacheEntry *ce);

int
fcache_get_attr (FCacheEntry *e, CredCacheEntry *ce);

int
fcache_get_data (FCacheEntry *e, CredCacheEntry *ce);

int
followmountpoint (VenusFid *fid, VenusFid *parent, CredCacheEntry **ce);

void
fcache_status (FILE *f);

#endif /* _FCACHE_H_ */
